from typing import Callable

from django.http import HttpRequest
from django.utils.timezone import now

from axes.exceptions import AxesSignalPermissionDenied
from axes.helpers import (
    get_client_ip_address,
    get_client_user_agent,
    get_client_path_info,
    get_client_http_accept,
    get_lockout_response,
)
from axes.request import AxesHttpRequest


class AxesMiddleware:
    """
    Middleware that maps lockout signals into readable HTTP 403 Forbidden responses.

    Without this middleware the backend returns HTTP 403 errors with the
    ``django.views.defaults.permission_denied`` view that renders the ``403.html``
    template from the root template directory if found.
    This middleware uses the ``axes.helpers.get_lockout_response`` handler
    for returning a context aware lockout message to the end user.

    To customize the error rendering, you can subclass this middleware
    and change the ``process_exception`` handler to your own liking.
    """

    def __init__(self, get_response: Callable):
        self.get_response = get_response

    def __call__(self, request: HttpRequest):
        self.update_request(request)
        return self.get_response(request)

    def update_request(self, request: HttpRequest):
        """
        Update given Django ``HttpRequest`` with necessary attributes
        before passing it on the ``get_response`` for further
        Django middleware and view processing.
        """

        request.axes_attempt_time = now()
        request.axes_ip_address = get_client_ip_address(request)
        request.axes_user_agent = get_client_user_agent(request)
        request.axes_path_info = get_client_path_info(request)
        request.axes_http_accept = get_client_http_accept(request)

    def process_exception(self, request: AxesHttpRequest, exception):  # pylint: disable=inconsistent-return-statements
        """
        Exception handler that processes exceptions raised by the Axes signal handler when request fails with login.

        Only ``axes.exceptions.AxesSignalPermissionDenied`` exception is handled by this middleware.
        """

        if isinstance(exception, AxesSignalPermissionDenied):
            return get_lockout_response(request)



from unittest.mock import MagicMock, patch

from django.test import override_settings
from django.utils.timezone import timedelta

from axes.conf import settings
from axes.handlers.proxy import AxesProxyHandler
from axes.tests.base import AxesTestCase
from axes.helpers import get_client_str


@override_settings(AXES_HANDLER='axes.handlers.base.AxesHandler')
class AxesHandlerTestCase(AxesTestCase):
    def test_base_handler_raises_on_undefined_is_allowed_to_authenticate(self):
        with self.assertRaises(NotImplementedError):
            AxesProxyHandler.is_allowed(self.request, {})

    @override_settings(AXES_IP_BLACKLIST=['127.0.0.1'])
    def test_is_allowed_with_blacklisted_ip_address(self):
        self.assertFalse(AxesProxyHandler.is_allowed(self.request))

    @override_settings(
        AXES_NEVER_LOCKOUT_WHITELIST=True,
        AXES_IP_WHITELIST=['127.0.0.1'],
    )
    def test_is_allowed_with_whitelisted_ip_address(self):
        self.assertTrue(AxesProxyHandler.is_allowed(self.request))

    @override_settings(AXES_NEVER_LOCKOUT_GET=True)
    def test_is_allowed_with_whitelisted_method(self):
        self.request.method = 'GET'
        self.assertTrue(AxesProxyHandler.is_allowed(self.request))

    @override_settings(AXES_LOCK_OUT_AT_FAILURE=False)
    def test_is_allowed_no_lock_out(self):
        self.assertTrue(AxesProxyHandler.is_allowed(self.request))


class AxesProxyHandlerTestCase(AxesTestCase):
    def setUp(self):
        self.sender = MagicMock()
        self.credentials = MagicMock()
        self.request = MagicMock()
        self.user = MagicMock()
        self.instance = MagicMock()

    @patch('axes.handlers.proxy.AxesProxyHandler.implementation', None)
    def test_setting_changed_signal_triggers_handler_reimport(self):
            self.assertIsNone(AxesProxyHandler.implementation)

            with self.settings(AXES_HANDLER='axes.handlers.database.AxesDatabaseHandler'):
                self.assertIsNotNone(AxesProxyHandler.implementation)

    @patch('axes.handlers.proxy.AxesProxyHandler.implementation')
    def test_user_login_failed(self, handler):
        self.assertFalse(handler.user_login_failed.called)
        AxesProxyHandler.user_login_failed(self.sender, self.credentials, self.request)
        self.assertTrue(handler.user_login_failed.called)

    @patch('axes.handlers.proxy.AxesProxyHandler.implementation')
    def test_user_logged_in(self, handler):
        self.assertFalse(handler.user_logged_in.called)
        AxesProxyHandler.user_logged_in(self.sender, self.request, self.user)
        self.assertTrue(handler.user_logged_in.called)

    @patch('axes.handlers.proxy.AxesProxyHandler.implementation')
    def test_user_logged_out(self, handler):
        self.assertFalse(handler.user_logged_out.called)
        AxesProxyHandler.user_logged_out(self.sender, self.request, self.user)
        self.assertTrue(handler.user_logged_out.called)

    @patch('axes.handlers.proxy.AxesProxyHandler.implementation')
    def test_post_save_access_attempt(self, handler):
        self.assertFalse(handler.post_save_access_attempt.called)
        AxesProxyHandler.post_save_access_attempt(self.instance)
        self.assertTrue(handler.post_save_access_attempt.called)

    @patch('axes.handlers.proxy.AxesProxyHandler.implementation')
    def test_post_delete_access_attempt(self, handler):
        self.assertFalse(handler.post_delete_access_attempt.called)
        AxesProxyHandler.post_delete_access_attempt(self.instance)
        self.assertTrue(handler.post_delete_access_attempt.called)


class AxesHandlerBaseTestCase(AxesTestCase):
    def check_whitelist(self, log):
        with override_settings(
            AXES_NEVER_LOCKOUT_WHITELIST=True,
            AXES_IP_WHITELIST=[self.ip_address],
        ):
            AxesProxyHandler.user_login_failed(sender=None, request=self.request, credentials=self.credentials)
            client_str = get_client_str(self.username, self.ip_address, self.user_agent, self.path_info)
            log.info.assert_called_with('AXES: Login failed from whitelisted client %s.', client_str)

    def check_empty_request(self, log, handler):
        AxesProxyHandler.user_login_failed(sender=None, credentials={}, request=None)
        log.error.assert_called_with(f'AXES: {handler}.user_login_failed does not function without a request.')


@override_settings(
    AXES_HANDLER='axes.handlers.database.AxesDatabaseHandler',
    AXES_COOLOFF_TIME=timedelta(seconds=1),
    AXES_RESET_ON_SUCCESS=True,
)
class AxesDatabaseHandlerTestCase(AxesHandlerBaseTestCase):
    @override_settings(AXES_RESET_ON_SUCCESS=True)
    def test_handler(self):
        self.check_handler()

    @override_settings(AXES_RESET_ON_SUCCESS=False)
    def test_handler_without_reset(self):
        self.check_handler()

    @patch('axes.handlers.database.log')
    def test_empty_request(self, log):
        self.check_empty_request(log, 'AxesDatabaseHandler')

    @patch('axes.handlers.database.log')
    def test_whitelist(self, log):
        self.check_whitelist(log)

    @patch('axes.handlers.database.is_user_attempt_whitelisted', return_value=True)
    def test_user_whitelisted(self, is_whitelisted):
        self.assertFalse(AxesProxyHandler().is_locked(self.request, self.credentials))
        self.assertEqual(1, is_whitelisted.call_count)


@override_settings(
    AXES_HANDLER='axes.handlers.cache.AxesCacheHandler',
    AXES_COOLOFF_TIME=timedelta(seconds=1),
)
class AxesCacheHandlerTestCase(AxesHandlerBaseTestCase):
    @override_settings(AXES_RESET_ON_SUCCESS=True)
    def test_handler(self):
        self.check_handler()

    @override_settings(AXES_RESET_ON_SUCCESS=False)
    def test_handler_without_reset(self):
        self.check_handler()

    @patch('axes.handlers.cache.log')
    def test_empty_request(self, log):
        self.check_empty_request(log, 'AxesCacheHandler')

    @patch('axes.handlers.cache.log')
    def test_whitelist(self, log):
        self.check_whitelist(log)


@override_settings(
    AXES_HANDLER='axes.handlers.dummy.AxesDummyHandler',
)
class AxesDummyHandlerTestCase(AxesHandlerBaseTestCase):
    def test_handler(self):
        for _ in range(settings.AXES_FAILURE_LIMIT):
            self.login()

        self.check_login()



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('axes', '0003_auto_20160322_0929'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='accessattempt',
            options={'verbose_name': 'access attempt', 'verbose_name_plural': 'access attempts'},
        ),
        migrations.AlterModelOptions(
            name='accesslog',
            options={'verbose_name': 'access log', 'verbose_name_plural': 'access logs'},
        ),
        migrations.AlterField(
            model_name='accessattempt',
            name='attempt_time',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Attempt Time'),
        ),
        migrations.AlterField(
            model_name='accessattempt',
            name='user_agent',
            field=models.CharField(db_index=True, max_length=255, verbose_name='User Agent'),
        ),
        migrations.AlterField(
            model_name='accessattempt',
            name='username',
            field=models.CharField(db_index=True, max_length=255, null=True, verbose_name='Username'),
        ),
        migrations.AlterField(
            model_name='accesslog',
            name='attempt_time',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Attempt Time'),
        ),
        migrations.AlterField(
            model_name='accesslog',
            name='logout_time',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Logout Time'),
        ),
        migrations.AlterField(
            model_name='accesslog',
            name='user_agent',
            field=models.CharField(db_index=True, max_length=255, verbose_name='User Agent'),
        ),
        migrations.AlterField(
            model_name='accesslog',
            name='username',
            field=models.CharField(db_index=True, max_length=255, null=True, verbose_name='Username'),
        ),
    ]



from unittest.mock import patch, MagicMock

from axes.backends import AxesBackend
from axes.exceptions import AxesBackendRequestParameterRequired, AxesBackendPermissionDenied
from axes.tests.base import AxesTestCase


class BackendTestCase(AxesTestCase):
    def test_authenticate_raises_on_missing_request(self):
        request = None

        with self.assertRaises(AxesBackendRequestParameterRequired):
            AxesBackend().authenticate(request)

    @patch('axes.handlers.proxy.AxesProxyHandler.is_allowed', return_value=False)
    def test_authenticate_raises_on_locked_request(self, _):
        request = MagicMock()

        with self.assertRaises(AxesBackendPermissionDenied):
            AxesBackend().authenticate(request)



from django.core.checks import Error, Tags, register

from axes.conf import settings


class Messages:
    CACHE_INVALID = 'invalid cache configuration for settings.AXES_CACHE'
    MIDDLEWARE_INVALID = 'axes.middleware.AxesMiddleware not in settings.MIDDLEWARE'
    BACKEND_INVALID = 'axes.backends.AxesBackend not in settings.AUTHENTICATION_BACKENDS'


class Hints:
    CACHE_INVALID = (
        'django-axes does not work properly with LocMemCache as the cache backend.'
        ' Please check the django-axes documentation and reconfigure settings.AXES_CACHE.'
    )
    MIDDLEWARE_INVALID = (
        'django-axes does not work properly without axes.middleware.AxesMiddleware in settings.MIDDLEWARE.'
        ' Please check the django-axes documentation and reconfigure settings.MIDDLEWARE.'
    )
    BACKEND_INVALID = (
        'django-axes does not work properly without axes.backends.AxesBackend in settings.AUTHENTICATION_BACKENDS.'
        ' Please check the django-axes documentation and reconfigure settings.AUTHENTICATION_BACKENDS.'
        ' Please note that the backend name was changed from AxesModelBackend to AxesBackend in django-axes version 5.'
    )


class Codes:
    CACHE_INVALID = 'axes.E001'
    MIDDLEWARE_INVALID = 'axes.E002'
    BACKEND_INVALID = 'axes.E003'


@register(Tags.compatibility, Tags.caches)
def axes_cache_check(app_configs, **kwargs):  # pylint: disable=unused-argument
    axes_handler = getattr(settings, 'AXES_HANDLER', '')

    axes_cache_key = getattr(settings, 'AXES_CACHE', 'default')
    axes_cache_config = settings.CACHES.get(axes_cache_key, {})
    axes_cache_backend = axes_cache_config.get('BACKEND', '')

    axes_cache_backend_incompatible = [
        'django.core.cache.backends.dummy.DummyCache',
        'django.core.cache.backends.locmem.LocMemCache',
        'django.core.cache.backends.filebased.FileBasedCache',
    ]

    errors = []

    if axes_handler == 'axes.handlers.cache.AxesCacheHandler':
        if axes_cache_backend in axes_cache_backend_incompatible:
            errors.append(Error(
                msg=Messages.CACHE_INVALID,
                hint=Hints.CACHE_INVALID,
                obj=settings.CACHES,
                id=Codes.CACHE_INVALID,
            ))

    return errors


@register(Tags.compatibility)
def axes_middleware_check(app_configs, **kwargs):  # pylint: disable=unused-argument
    errors = []

    if 'axes.middleware.AxesMiddleware' not in settings.MIDDLEWARE:
        errors.append(Error(
            msg=Messages.MIDDLEWARE_INVALID,
            hint=Hints.MIDDLEWARE_INVALID,
            obj=settings.MIDDLEWARE,
            id=Codes.MIDDLEWARE_INVALID,
        ))

    return errors


@register(Tags.compatibility)
def axes_backend_check(app_configs, **kwargs):  # pylint: disable=unused-argument
    errors = []

    if 'axes.backends.AxesBackend' not in settings.AUTHENTICATION_BACKENDS:
        errors.append(Error(
            msg=Messages.BACKEND_INVALID,
            hint=Hints.BACKEND_INVALID,
            obj=settings.AUTHENTICATION_BACKENDS,
            id=Codes.BACKEND_INVALID,
        ))

    return errors



from logging import getLogger

from axes.conf import settings
from axes.exceptions import AxesSignalPermissionDenied
from axes.request import AxesHttpRequest
from axes.handlers.base import AxesHandler
from axes.signals import user_locked_out
from axes.helpers import (
    get_cache,
    get_cache_timeout,
    get_client_cache_key,
    get_client_str,
    get_client_username,
    get_credentials,
)

log = getLogger(settings.AXES_LOGGER)


class AxesCacheHandler(AxesHandler):  # pylint: disable=too-many-locals
    """
    Signal handler implementation that records user login attempts to cache and locks users out if necessary.
    """

    def __init__(self):
        self.cache = get_cache()
        self.cache_timeout = get_cache_timeout()

    def get_failures(self, request: AxesHttpRequest, credentials: dict = None) -> int:
        cache_key = get_client_cache_key(request, credentials)
        return self.cache.get(cache_key, default=0)

    def user_login_failed(
            self,
            sender,
            credentials: dict,
            request: AxesHttpRequest = None,
            **kwargs
    ):  # pylint: disable=too-many-locals
        """
        When user login fails, save attempt record in cache and lock user out if necessary.

        :raises AxesSignalPermissionDenied: if user should be locked out.
        """

        if request is None:
            log.error('AXES: AxesCacheHandler.user_login_failed does not function without a request.')
            return

        username = get_client_username(request, credentials)
        client_str = get_client_str(username, request.axes_ip_address, request.axes_user_agent, request.axes_path_info)

        if self.is_whitelisted(request, credentials):
            log.info('AXES: Login failed from whitelisted client %s.', client_str)
            return

        failures_since_start = 1 + self.get_failures(request, credentials)

        if failures_since_start > 1:
            log.warning(
                'AXES: Repeated login failure by %s. Count = %d of %d. Updating existing record in the cache.',
                client_str,
                failures_since_start,
                settings.AXES_FAILURE_LIMIT,
            )
        else:
            log.warning(
                'AXES: New login failure by %s. Creating new record in the cache.',
                client_str,
            )

        cache_key = get_client_cache_key(request, credentials)
        self.cache.set(cache_key, failures_since_start, self.cache_timeout)

        if failures_since_start >= settings.AXES_FAILURE_LIMIT:
            log.warning('AXES: Locking out %s after repeated login failures.', client_str)

            user_locked_out.send(
                'axes',
                request=request,
                username=username,
                ip_address=request.axes_ip_address,
            )

            raise AxesSignalPermissionDenied('Locked out due to repeated login failures.')

    def user_logged_in(self, sender, request: AxesHttpRequest, user, **kwargs):  # pylint: disable=unused-argument
        """
        When user logs in, update the AccessLog related to the user.
        """

        username = user.get_username()
        credentials = get_credentials(username)
        client_str = get_client_str(username, request.axes_ip_address, request.axes_user_agent, request.axes_path_info)

        log.info('AXES: Successful login by %s.', client_str)

        if settings.AXES_RESET_ON_SUCCESS:
            cache_key = get_client_cache_key(request, credentials)
            failures_since_start = self.cache.get(cache_key, default=0)
            self.cache.delete(cache_key)
            log.info('AXES: Deleted %d failed login attempts by %s from cache.', failures_since_start, client_str)

    def user_logged_out(self, sender, request: AxesHttpRequest, user, **kwargs):
        username = user.get_username()
        client_str = get_client_str(username, request.axes_ip_address, request.axes_user_agent, request.axes_path_info)

        log.info('AXES: Successful logout by %s.', client_str)



# Generated by Django 2.0.4 on 2019-03-13 08:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('axes', '0005_remove_accessattempt_trusted'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='accesslog',
            name='trusted',
        ),
    ]



from django.core.management.base import BaseCommand

from axes.utils import reset


class Command(BaseCommand):
    help = 'Reset all access attempts and lockouts'

    def handle(self, *args, **options):  # pylint: disable=unused-argument
        count = reset()

        if count:
            self.stdout.write(f'{count} attempts removed.')
        else:
            self.stdout.write('No attempts found.')



#!/usr/bin/env python

from setuptools import setup, find_packages

from axes import get_version

setup(
    name='django-axes',
    version=get_version(),
    description='Keep track of failed login attempts in Django-powered sites.',
    long_description='\n'.join([
        open('README.rst', encoding='utf-8').read(),
        open('CHANGES.rst', encoding='utf-8').read(),
    ]),
    keywords='authentication django pci security',
    author=', '.join([
        'Josh VanderLinden',
        'Philip Neustrom',
        'Michael Blume',
        'Alex Clark',
        'Camilo Nova',
        'Aleksi Hakli',
    ]),
    maintainer='Jazzband',
    maintainer_email='security@jazzband.co',
    url='https://github.com/jazzband/django-axes',
    project_urls={
        'Documentation': 'https://django-axes.readthedocs.io/',
        'Source': 'https://github.com/jazzband/django-axes',
        'Tracker': 'https://github.com/jazzband/django-axes/issues',
    },
    license='MIT',
    package_dir={'axes': 'axes'},
    python_requires='~=3.6',
    install_requires=[
        'django>=1.11',
        'django-appconf>=1.0.3',
        'django-ipware>=2.0.2',
    ],
    include_package_data=True,
    packages=find_packages(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: Log Analysis',
        'Topic :: Security',
        'Topic :: System :: Logging',
    ],
    zip_safe=False,
)



from django.core.exceptions import PermissionDenied


class AxesPermissionDenied(PermissionDenied):
    """
    Base class for permission denied errors raised by axes specifically for easier debugging.

    Two different types of errors are used because of the behaviour Django has:

    - If an authentication backend raises a PermissionDenied error the authentication flow is aborted.
    - If another component raises a PermissionDenied error a HTTP 403 Forbidden response is returned.
    """

    pass


class AxesSignalPermissionDenied(AxesPermissionDenied):
    """
    Raised by signal handler on failed authentication attempts to send user a HTTP 403 Forbidden status code.
    """

    pass


class AxesBackendPermissionDenied(AxesPermissionDenied):
    """
    Raised by authentication backend on locked out requests to stop the Django authentication flow.
    """

    pass


class AxesBackendRequestParameterRequired(ValueError):
    """
    Raised by authentication backend on invalid or missing request parameter value.
    """

    pass



from logging import getLogger

from django.utils.module_loading import import_string

from axes.conf import settings
from axes.handlers.base import AxesHandler
from axes.request import AxesHttpRequest

log = getLogger(settings.AXES_LOGGER)


class AxesProxyHandler(AxesHandler):
    """
    Proxy interface for configurable Axes signal handler class.

    If you wish to implement a custom version of this handler,
    you can override the settings.AXES_HANDLER configuration string
    with a class that implements a compatible interface and methods.

    Defaults to using axes.handlers.proxy.AxesProxyHandler if not overridden.
    Refer to axes.handlers.proxy.AxesProxyHandler for default implementation.
    """

    implementation = None  # type: AxesHandler

    @classmethod
    def get_implementation(cls, force: bool = False) -> AxesHandler:
        """
        Fetch and initialize configured handler implementation and memoize it to avoid reinitialization.

        This method is re-entrant and can be called multiple times from e.g. Django application loader.
        """

        if force or not cls.implementation:
            cls.implementation = import_string(settings.AXES_HANDLER)()
        return cls.implementation

    @classmethod
    def is_locked(cls, request: AxesHttpRequest, credentials: dict = None) -> bool:
        return cls.get_implementation().is_locked(request, credentials)

    @classmethod
    def is_allowed(cls, request: AxesHttpRequest, credentials: dict = None) -> bool:
        return cls.get_implementation().is_allowed(request, credentials)

    @classmethod
    def user_login_failed(cls, sender, credentials: dict, request: AxesHttpRequest = None, **kwargs):
        return cls.get_implementation().user_login_failed(sender, credentials, request, **kwargs)

    @classmethod
    def user_logged_in(cls, sender, request: AxesHttpRequest, user, **kwargs):
        return cls.get_implementation().user_logged_in(sender, request, user, **kwargs)

    @classmethod
    def user_logged_out(cls, sender, request: AxesHttpRequest, user, **kwargs):
        return cls.get_implementation().user_logged_out(sender, request, user, **kwargs)

    @classmethod
    def post_save_access_attempt(cls, instance, **kwargs):
        return cls.get_implementation().post_save_access_attempt(instance, **kwargs)

    @classmethod
    def post_delete_access_attempt(cls, instance, **kwargs):
        return cls.get_implementation().post_delete_access_attempt(instance, **kwargs)



from logging import getLogger

from django import apps

from axes import get_version
from axes.conf import settings

log = getLogger(settings.AXES_LOGGER)


class AppConfig(apps.AppConfig):
    name = 'axes'
    logging_initialized = False

    @classmethod
    def initialize(cls):
        """
        Initialize Axes logging and show version information.

        This method is re-entrant and can be called multiple times.
        It displays version information exactly once at application startup.
        """

        if cls.logging_initialized:
            return
        cls.logging_initialized = True

        if not settings.AXES_VERBOSE:
            return

        log.info('AXES: BEGIN LOG')
        log.info('AXES: Using django-axes %s', get_version())

        if settings.AXES_ONLY_USER_FAILURES:
            log.info('AXES: blocking by username only.')
        elif settings.AXES_LOCK_OUT_BY_COMBINATION_USER_AND_IP:
            log.info('AXES: blocking by combination of username and IP.')
        else:
            log.info('AXES: blocking by IP only.')

    def ready(self):
        self.initialize()

        from axes import signals  # noqa



from unittest.mock import patch, MagicMock

from django.http import HttpResponse

from axes.exceptions import AxesSignalPermissionDenied
from axes.middleware import AxesMiddleware
from axes.tests.base import AxesTestCase


class MiddlewareTestCase(AxesTestCase):
    SUCCESS_RESPONSE = HttpResponse(status=200, content='Dispatched')
    LOCKOUT_RESPONSE = HttpResponse(status=403, content='Locked out')

    def setUp(self):
        self.request = MagicMock()
        self.get_response = MagicMock()

    @patch('axes.middleware.get_lockout_response', return_value=LOCKOUT_RESPONSE)
    def test_process_exception_axes(self, _):
        exception = AxesSignalPermissionDenied()
        response = AxesMiddleware(self.get_response).process_exception(self.request, exception)
        self.assertEqual(response, self.LOCKOUT_RESPONSE)

    @patch('axes.middleware.get_lockout_response', return_value=LOCKOUT_RESPONSE)
    def test_process_exception_other(self, _):
        exception = Exception()
        response = AxesMiddleware(self.get_response).process_exception(self.request, exception)
        self.assertEqual(response, None)
